@@ -34,12 +34,11 @@ use crate::commands::project::add::{AddTarget, PythonTarget};
 use crate::commands::project::install_target::InstallTarget;
 use crate::commands::project::lock::LockMode;
 use crate::commands::project::{
-    MissingLockfileSource, ProjectEnvironment, ProjectError, ProjectInterpreter, UniversalState,
-    default_dependency_groups,
+    ProjectEnvironment, ProjectError, ProjectInterpreter, UniversalState, default_dependency_groups,
 };
 use crate::commands::{ExitStatus, diagnostics, project};
 use crate::printer::Printer;
-use crate::settings::{LockCheck, ResolverInstallerSettings};
+use crate::settings::{FrozenSource, LockCheck, ResolverInstallerSettings};
 
 /// Display version information for uv itself (`uv self version`)
 pub(crate) fn self_version(
@@ -65,7 +64,7 @@ pub(crate) async fn project_version(
     explicit_project: bool,
     dry_run: bool,
     lock_check: LockCheck,
-    frozen: bool,
+    frozen: Option<FrozenSource>,
     active: Option<bool>,
     no_sync: bool,
     python: Option<String>,
@@ -94,27 +93,30 @@ pub(crate) async fn project_version(
 
     // Short-circuit early for a frozen read
     let is_read_only = value.is_none() && bump.is_empty();
-    if frozen && is_read_only {
-        return Box::pin(print_frozen_version(
-            project,
-            &name,
-            project_dir,
-            active,
-            python,
-            install_mirrors,
-            &settings,
-            client_builder,
-            python_preference,
-            python_downloads,
-            concurrency,
-            no_config,
-            cache,
-            short,
-            output_format,
-            printer,
-            preview,
-        ))
-        .await;
+    if let Some(frozen_source) = frozen {
+        if is_read_only {
+            return Box::pin(print_frozen_version(
+                project,
+                &name,
+                project_dir,
+                frozen_source,
+                active,
+                python,
+                install_mirrors,
+                &settings,
+                client_builder,
+                python_preference,
+                python_downloads,
+                concurrency,
+                no_config,
+                cache,
+                short,
+                output_format,
+                printer,
+                preview,
+            ))
+            .await;
+        }
     }
 
     let mut toml = PyProjectTomlMut::from_toml(
@@ -422,6 +424,7 @@ async fn print_frozen_version(
     project: VirtualProject,
     name: &PackageName,
     project_dir: &Path,
+    frozen_source: FrozenSource,
     active: Option<bool>,
     python: Option<String>,
     install_mirrors: PythonInstallMirrors,
@@ -465,7 +468,7 @@ async fn print_frozen_version(
     // Lock and sync the environment, if necessary.
     let lock = match Box::pin(
         project::lock::LockOperation::new(
-            LockMode::Frozen(MissingLockfileSource::frozen()),
+            LockMode::Frozen(frozen_source.into()),
             &settings.resolver,
             &client_builder,
             &state,
@@ -518,7 +521,7 @@ async fn lock_and_sync(
     project: VirtualProject,
     project_dir: &Path,
     lock_check: LockCheck,
-    frozen: bool,
+    frozen: Option<FrozenSource>,
     active: Option<bool>,
     no_sync: bool,
     python: Option<String>,
@@ -535,7 +538,7 @@ async fn lock_and_sync(
     preview: Preview,
 ) -> Result<ExitStatus> {
     // If frozen, don't touch the lock or sync at all
-    if frozen {
+    if frozen.is_some() {
         return Ok(ExitStatus::Success);
     }
 
