@@ -4,6 +4,7 @@ use std::path::Path;
 use anyhow::Result;
 
 use owo_colors::OwoColorize;
+use uv_fs::Simplified;
 use uv_preview::{Preview, PreviewFeatures};
 use uv_warnings::warn_user;
 use uv_workspace::{DiscoveryOptions, Workspace, WorkspaceCache};
@@ -14,6 +15,7 @@ use crate::printer::Printer;
 /// List workspace members
 pub(crate) async fn list(
     project_dir: &Path,
+    paths: bool,
     preview: Preview,
     printer: Printer,
 ) -> Result<ExitStatus> {
@@ -28,8 +30,16 @@ pub(crate) async fn list(
     let workspace =
         Workspace::discover(project_dir, &DiscoveryOptions::default(), &workspace_cache).await?;
 
-    for name in workspace.packages().keys() {
-        writeln!(printer.stdout(), "{}", name.cyan())?;
+    for (name, member) in workspace.packages() {
+        if paths {
+            writeln!(
+                printer.stdout(),
+                "{}",
+                member.root().simplified_display().cyan()
+            )?;
+        } else {
+            writeln!(printer.stdout(), "{}", name.cyan())?;
+        }
     }
 
     Ok(ExitStatus::Success)
