@@ -576,6 +576,20 @@ fn help_subsubcommand() {
               
               If multiple Python versions are requested, uv will exit with an error.
 
+          --compile-bytecode
+              Compile Python's standard library to bytecode after installation.
+              
+              By default, uv does not compile Python (`.py`) files to bytecode (`__pycache__/*.pyc`);
+              instead, compilation is performed lazily the first time a module is imported. For
+              use-cases in which start time is important, such as CLI applications and Docker
+              containers, this option can be enabled to trade longer installation times and some
+              additional disk space for faster start times.
+              
+              When enabled, uv will process the Python version's `stdlib` directory. It will ignore any
+              compilation errors.
+              
+              [env: UV_COMPILE_BYTECODE=]
+
     Cache options:
       -n, --no-cache
               Avoid reading from or writing to the cache, instead using a temporary directory for the
@@ -827,6 +841,9 @@ fn help_flag_subsubcommand() {
               Upgrade existing Python installations to the latest patch version
           --default
               Use as the default Python version
+          --compile-bytecode
+              Compile Python's standard library to bytecode after installation [env:
+              UV_COMPILE_BYTECODE=]
 
     Cache options:
       -n, --no-cache               Avoid reading from or writing to the cache, instead using a temporary
