@@ -23,7 +23,7 @@ fn username_password_no_longer_supported() {
         .arg("dummy")
         .arg("--publish-url")
         .arg("https://test.pypi.org/legacy/")
-        .arg("../../scripts/links/ok-1.0.0-py3-none-any.whl"), @r###"
+        .arg("../../test/links/ok-1.0.0-py3-none-any.whl"), @r###"
     success: false
     exit_code: 2
     ----- stdout -----
@@ -31,7 +31,7 @@ fn username_password_no_longer_supported() {
     ----- stderr -----
     Publishing 1 file to https://test.pypi.org/legacy/
     Uploading ok-1.0.0-py3-none-any.whl ([SIZE])
-    error: Failed to publish `../../scripts/links/ok-1.0.0-py3-none-any.whl` to https://test.pypi.org/legacy/
+    error: Failed to publish `../../test/links/ok-1.0.0-py3-none-any.whl` to https://test.pypi.org/legacy/
       Caused by: Upload failed with status code 403 Forbidden. Server says: 403 Username/Password authentication is no longer supported. Migrate to API Tokens or Trusted Publishers instead. See https://test.pypi.org/help/#apitoken and https://test.pypi.org/help/#trusted-publishers
     "###
     );
@@ -48,7 +48,7 @@ fn invalid_token() {
         .arg("dummy")
         .arg("--publish-url")
         .arg("https://test.pypi.org/legacy/")
-        .arg("../../scripts/links/ok-1.0.0-py3-none-any.whl"), @r###"
+        .arg("../../test/links/ok-1.0.0-py3-none-any.whl"), @r###"
     success: false
     exit_code: 2
     ----- stdout -----
@@ -56,7 +56,7 @@ fn invalid_token() {
     ----- stderr -----
     Publishing 1 file to https://test.pypi.org/legacy/
     Uploading ok-1.0.0-py3-none-any.whl ([SIZE])
-    error: Failed to publish `../../scripts/links/ok-1.0.0-py3-none-any.whl` to https://test.pypi.org/legacy/
+    error: Failed to publish `../../test/links/ok-1.0.0-py3-none-any.whl` to https://test.pypi.org/legacy/
       Caused by: Upload failed with status code 403 Forbidden. Server says: 403 Invalid or non-existent authentication information. See https://test.pypi.org/help/#invalid-auth for more information.
     "###
     );
@@ -76,7 +76,7 @@ fn mixed_credentials() {
         .arg("https://test.pypi.org/legacy/")
         .arg("--trusted-publishing")
         .arg("always")
-        .arg("../../scripts/links/ok-1.0.0-py3-none-any.whl")
+        .arg("../../test/links/ok-1.0.0-py3-none-any.whl")
         // Emulate CI
         .env(EnvVars::GITHUB_ACTIONS, "true"), @r###"
     success: false
@@ -100,7 +100,7 @@ fn missing_trusted_publishing_permission() {
         .arg("https://test.pypi.org/legacy/")
         .arg("--trusted-publishing")
         .arg("always")
-        .arg("../../scripts/links/ok-1.0.0-py3-none-any.whl")
+        .arg("../../test/links/ok-1.0.0-py3-none-any.whl")
         // Emulate CI
         .env(EnvVars::GITHUB_ACTIONS, "true"), @r"
     success: false
@@ -126,7 +126,7 @@ fn no_credentials() {
     uv_snapshot!(context.filters(), context.publish()
         .arg("--publish-url")
         .arg("https://test.pypi.org/legacy/")
-        .arg("../../scripts/links/ok-1.0.0-py3-none-any.whl")
+        .arg("../../test/links/ok-1.0.0-py3-none-any.whl")
         // Emulate CI
         .env(EnvVars::GITHUB_ACTIONS, "true"), @r"
     success: false
@@ -141,7 +141,7 @@ fn no_credentials() {
       Caused by: GitHub Actions detection error
       Caused by: insufficient permissions: missing ACTIONS_ID_TOKEN_REQUEST_URL
     Uploading ok-1.0.0-py3-none-any.whl ([SIZE])
-    error: Failed to publish `../../scripts/links/ok-1.0.0-py3-none-any.whl` to https://test.pypi.org/legacy/
+    error: Failed to publish `../../test/links/ok-1.0.0-py3-none-any.whl` to https://test.pypi.org/legacy/
       Caused by: Failed to send POST request
       Caused by: Missing credentials for https://test.pypi.org/legacy/
     "
@@ -211,7 +211,7 @@ fn check_keyring_behaviours() {
         .arg(
             context
                 .workspace_root
-                .join("scripts")
+                .join("test")
                 .join("packages")
                 .join("keyring_test_plugin"),
         )
@@ -230,7 +230,7 @@ fn check_keyring_behaviours() {
         .arg("https://test.pypi.org/simple/")
         .arg("--publish-url")
         .arg("https://test.pypi.org/legacy/?ok")
-        .arg("../../scripts/links/ok-1.0.0-py3-none-any.whl")
+        .arg("../../test/links/ok-1.0.0-py3-none-any.whl")
         .env(EnvVars::PATH, venv_bin_path(&context.venv)), @r###"
     success: false
     exit_code: 2
@@ -239,7 +239,7 @@ fn check_keyring_behaviours() {
     ----- stderr -----
     Publishing 1 file to https://test.pypi.org/legacy/?ok
     Uploading ok-1.0.0-py3-none-any.whl ([SIZE])
-    error: Failed to publish `../../scripts/links/ok-1.0.0-py3-none-any.whl` to https://test.pypi.org/legacy/?ok
+    error: Failed to publish `../../test/links/ok-1.0.0-py3-none-any.whl` to https://test.pypi.org/legacy/?ok
       Caused by: Upload failed with status code 403 Forbidden. Server says: 403 Username/Password authentication is no longer supported. Migrate to API Tokens or Trusted Publishers instead. See https://test.pypi.org/help/#apitoken and https://test.pypi.org/help/#trusted-publishers
     "###
     );
@@ -254,7 +254,7 @@ fn check_keyring_behaviours() {
         .arg("subprocess")
         .arg("--publish-url")
         .arg("https://test.pypi.org/legacy/?ok")
-        .arg("../../scripts/links/ok-1.0.0-py3-none-any.whl")
+        .arg("../../test/links/ok-1.0.0-py3-none-any.whl")
         .env(EnvVars::PATH, venv_bin_path(&context.venv)),  @r###"
     success: false
     exit_code: 2
@@ -264,7 +264,7 @@ fn check_keyring_behaviours() {
     Publishing 1 file to https://test.pypi.org/legacy/?ok
     warning: Using `--keyring-provider` with a password or token and no check URL has no effect
     Uploading ok-1.0.0-py3-none-any.whl ([SIZE])
-    error: Failed to publish `../../scripts/links/ok-1.0.0-py3-none-any.whl` to https://test.pypi.org/legacy/?ok
+    error: Failed to publish `../../test/links/ok-1.0.0-py3-none-any.whl` to https://test.pypi.org/legacy/?ok
       Caused by: Upload failed with status code 403 Forbidden. Server says: 403 Username/Password authentication is no longer supported. Migrate to API Tokens or Trusted Publishers instead. See https://test.pypi.org/help/#apitoken and https://test.pypi.org/help/#trusted-publishers
     "###
     );
@@ -280,7 +280,7 @@ fn check_keyring_behaviours() {
         .arg("https://test.pypi.org/simple/")
         .arg("--publish-url")
         .arg("https://test.pypi.org/legacy/?ok")
-        .arg("../../scripts/links/ok-1.0.0-py3-none-any.whl")
+        .arg("../../test/links/ok-1.0.0-py3-none-any.whl")
         .env(EnvVars::PATH, venv_bin_path(&context.venv)), @r"
     success: false
     exit_code: 2
@@ -294,7 +294,7 @@ fn check_keyring_behaviours() {
     Uploading ok-1.0.0-py3-none-any.whl ([SIZE])
     Keyring request for dummy@https://test.pypi.org/legacy/?ok
     Keyring request for dummy@test.pypi.org
-    error: Failed to publish `../../scripts/links/ok-1.0.0-py3-none-any.whl` to https://test.pypi.org/legacy/?ok
+    error: Failed to publish `../../test/links/ok-1.0.0-py3-none-any.whl` to https://test.pypi.org/legacy/?ok
       Caused by: Upload failed with status code 403 Forbidden. Server says: 403 Username/Password authentication is no longer supported. Migrate to API Tokens or Trusted Publishers instead. See https://test.pypi.org/help/#apitoken and https://test.pypi.org/help/#trusted-publishers
     "
     );
@@ -308,7 +308,7 @@ fn check_keyring_behaviours() {
         .arg("subprocess")
         .arg("--publish-url")
         .arg("https://test.pypi.org/legacy/?ok")
-        .arg("../../scripts/links/ok-1.0.0-py3-none-any.whl")
+        .arg("../../test/links/ok-1.0.0-py3-none-any.whl")
         .env(EnvVars::KEYRING_TEST_CREDENTIALS, r#"{"https://test.pypi.org/legacy/?ok": {"dummy": "dummy"}}"#)
         .env(EnvVars::PATH, venv_bin_path(&context.venv)), @r"
     success: false
@@ -319,7 +319,7 @@ fn check_keyring_behaviours() {
     Publishing 1 file to https://test.pypi.org/legacy/?ok
     Keyring request for dummy@https://test.pypi.org/legacy/?ok
     Uploading ok-1.0.0-py3-none-any.whl ([SIZE])
-    error: Failed to publish `../../scripts/links/ok-1.0.0-py3-none-any.whl` to https://test.pypi.org/legacy/?ok
+    error: Failed to publish `../../test/links/ok-1.0.0-py3-none-any.whl` to https://test.pypi.org/legacy/?ok
       Caused by: Upload failed with status code 403 Forbidden. Server says: 403 Username/Password authentication is no longer supported. Migrate to API Tokens or Trusted Publishers instead. See https://test.pypi.org/help/#apitoken and https://test.pypi.org/help/#trusted-publishers
     "
     );
@@ -351,7 +351,7 @@ fn invalid_index() {
 
     let ok_wheel = current_dir()
         .unwrap()
-        .join("../../scripts/links/ok-1.0.0-py3-none-any.whl");
+        .join("../../test/links/ok-1.0.0-py3-none-any.whl");
 
     // No such index
     uv_snapshot!(context.filters(), context.publish()
@@ -537,7 +537,7 @@ async fn gitlab_trusted_publishing_pypi_id_token() {
         .arg("always")
         .arg("--publish-url")
         .arg(format!("{}/upload", server.uri()))
-        .arg("../../scripts/links/ok-1.0.0-py3-none-any.whl")
+        .arg("../../test/links/ok-1.0.0-py3-none-any.whl")
         .env(EnvVars::GITLAB_CI, "true")
         .env_remove(EnvVars::GITHUB_ACTIONS)
         .env(EnvVars::PYPI_ID_TOKEN, "gitlab-oidc-jwt"), @r"
@@ -591,7 +591,7 @@ async fn gitlab_trusted_publishing_testpypi_id_token() {
         .arg("always")
         .arg("--publish-url")
         .arg(format!("{}/upload", server.uri()))
-        .arg("../../scripts/links/ok-1.0.0-py3-none-any.whl")
+        .arg("../../test/links/ok-1.0.0-py3-none-any.whl")
         // Emulate GitLab CI with TESTPYPI_ID_TOKEN present
         .env(EnvVars::GITLAB_CI, "true")
         .env_remove(EnvVars::GITHUB_ACTIONS)
