@@ -45,8 +45,8 @@ def update_schemastore(schemastore: Path, *, root: Path) -> None:
         cwd=schemastore,
     )
 
-    # Run npm install
-    check_call(["npm", "install"], cwd=schemastore)
+    # Run npm ci
+    check_call(["npm", "ci", "--ignore-scripts"], cwd=schemastore)
 
     src = schemastore.joinpath("src")
 
